/*
 * Hull.h - class definition for hulls (collection of geometric
 *          objects).
 *
 * Copyright (C) 1992, Christoph Streit (streit@iam.unibe.ch)
 *                     University of Berne, Switzerland
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

#ifndef Hull_H
# define Hull_H

#include "GeoObject.h"
#include "BoundingBox.h"
#include "rcString.h"
#include "Polygon.h"
#include "table.h"

//___________________________________________________________ Hull

class Hull 
{
public:
  Hull();
  ~Hull();

  void addPrimitive(GeoObject*);
  long numPrimitives() const;
  int intersect(const Ray&, real&, GeoObject*&) const;
  PolygonList* convertToPolygonList(const BoundingBox&) const;

private:
  GeoObjectList* primitives;
};

typedef Hull* HullPtr;
declareTable(HullSymtab, rcString, HullPtr);

#endif // Hull_H
