/*
 * LineDevice.h - class definition simple line device driver.
 *
 * Copyright (C) 1992, Christoph Streit (streit@iam.unibe.ch)
 *                     University of Berne, Switzerland
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

#ifndef LineDevice_H
# define LineDevice_H

#include "DeviceDriver.h"
#include "BaseWindow.h"
#include "ViewTransform.h"

/*___________________________________________________________ LineDevice
 *
 * Simple line drawing device driver. Ignores width of lines 
 * and macro definitions. Uses BaseWindow class to draw to the screen.
 * LineDevice sets camera position if none is provided by the user.
 */

class LineDevice : public DeviceDriver 
{
public:
  LineDevice(BaseWindow*, Options*);
  ~LineDevice();

  void begin();
  void end(const BoundingBox&);
  void cylinder(const Vector&, const Vector&, real);
  void cone(const Vector&, real, const Vector&, real);
  void polygon(Polygon*);
  void sphere(const Vector&, real);

private:
  BaseWindow* w;
};

#endif // LineDevice_H

